package raytracer;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.filechooser.FileFilter;

import raytracer.internal.Utils;
import raytracer.internal.Constants;

/**
 * Created by IntelliJ IDEA.
 * User: Bogdy
 * Date: 06.05.2003
 * Time: 18:39:52
 * To change this template use Options | File Templates.
 */
public class ControlFrame extends JFrame implements ActionListener {
    private final String renderText = "Render";
    private final String stopText = "  Stop   ";
    private final String pauseText = "   Pause   ";
    private final String resumeText = " Resume ";

    private static final String[] sizeStrings = {"100x100", "300x300", "512x512", "800x800", "1024x768"};
    private static final String[] recurrenceStrings = {"Infinity", "Zero", "One", "Two", "Three", "Four", "Five"};

    private static ControlFrame frame = null;

    private File file = null;
    private File imageFile = null;
    private Renderer renderer = null;
    private JComboBox sizeCombo;
    private JComboBox recurrenceCombo;
    private JButton renderButton;
    private JButton pauseButton;
    private JProgressBar progress;
    private String synchObj = new String("_synchObj_MainFrame_");
    private JLabel console;
    private String consoleText;
    private JScrollPane scrollConsole;

    public ControlFrame() {
        super("Raytracer SS03");
        frame = this;
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // create layout
        getContentPane().setLayout(new BorderLayout());
        JPanel buttons = new JPanel();
        buttons.add(new JButton(new AbstractAction("Load") {
            public void actionPerformed(ActionEvent e) {
                load();
            }
        }));
        buttons.add(new JButton(new AbstractAction("Save") {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        }));
        sizeCombo = new JComboBox(sizeStrings);
        sizeCombo.addActionListener(this);
        buttons.add(new JLabel("Size:"));
        buttons.add(sizeCombo);
        recurrenceCombo = new JComboBox(recurrenceStrings);
        buttons.add(new JLabel("Recurrence:"));
        buttons.add(recurrenceCombo);
        renderButton = new JButton(new AbstractAction(renderText) {
            public void actionPerformed(ActionEvent e) {
                renderButton();
            }
        });
        buttons.add(renderButton);
        pauseButton = new JButton(new AbstractAction(pauseText) {
            public void actionPerformed(ActionEvent e) {
                pauseButton();
            }
        });
        pauseButton.setEnabled(false);
        buttons.add(pauseButton);
        buttons.add(new JButton(new AbstractAction("Quit") {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        }));
        getContentPane().add(buttons, BorderLayout.NORTH);
        renderer = new Renderer();
        getContentPane().add(renderer, BorderLayout.CENTER);

        JPanel southPanel = new JPanel(new BorderLayout());
        getContentPane().add(southPanel, BorderLayout.SOUTH);

        console = new JLabel();
        console.setAutoscrolls(true);
        console.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
        console.setForeground(Color.black);
        console.setBackground(Color.white);
        console.setOpaque(true);
        console.setEnabled(false);
        clearConsoleText();
        scrollConsole = new JScrollPane(console);
        scrollConsole.setWheelScrollingEnabled(true);
        scrollConsole.setAutoscrolls(true);
        scrollConsole.setEnabled(false);
        southPanel.add(scrollConsole, BorderLayout.CENTER);

        progress = new JProgressBar(JProgressBar.HORIZONTAL, 100);
        progress.setStringPainted(true);
        southPanel.add(progress, BorderLayout.SOUTH);

        sizeCombo.setSelectedIndex(1);
        renderFinished(Renderer.SUCCEEDED);
        pack();
        scrollConsole.setPreferredSize(new Dimension(getWidth(), 100));
        pack();
    }

    private void renderButton() {
        pauseButton.setText(pauseText);
        if (renderingRunning()) {
            pauseButton.setEnabled(false);
            renderer.cancelRendering();
        } else {
            pauseButton.setEnabled(true);
            reload();
            renderButton.setText(stopText);
            renderer.setRecursionDepth(recurrenceCombo.getSelectedIndex()-1);
            renderer.startRendering(this);
        }
    }

    private boolean renderingRunning() {
        return renderButton.getText().compareTo(stopText) == 0;
    }

    private void pauseButton() {
        // running
        if (pauseButton.getText().compareTo(pauseText) == 0) {
            pauseButton.setText(resumeText);
            renderer.pauseRendering();
        } else { // paused
            pauseButton.setText(pauseText);
            renderButton.setText(stopText);
            renderer.resumeRendering(this);
        }
    }

    public void setParams(String fileName, int width, int height, int recursionDepth) {
        // filename
        file = new File(fileName);
        reload();

        // recursion depth
        try {
            recurrenceCombo.setSelectedIndex(recursionDepth);
        } catch (Exception e) {}
        renderer.setRecursionDepth(recurrenceCombo.getSelectedIndex()-1);

        // size
        renderer.setSize(width, height);
        final String str = width+"x"+height;
        boolean found = false;
        for (int i = 0; i < sizeStrings.length; i++) {
            if (sizeStrings[i].compareTo(str) == 0) {
                found = true;
                sizeCombo.setSelectedIndex(i);
            }
        }
        if (!found) {
            sizeCombo.addItem(str);
            sizeCombo.setSelectedIndex(sizeCombo.getItemCount()-1);
        }
        pack();
        scrollConsole.setPreferredSize(new Dimension(getWidth(), 100));
        pack();
    }

    private void reload() {
        if (file == null) {
            addConsoleText("No loaded scene", Constants.failColor);
            JOptionPane.showMessageDialog(ControlFrame.this, "No file loaded !");
        } else {
            addConsoleText("Loading scene "+file.getName(), Constants.commentColor);
            renderer.setSceneName(file.getAbsolutePath());
            addConsoleText("Scene loaded", Constants.commentColor);
        }
    }

    private void load() {
        JFileChooser fileChooser = new JFileChooser(file);
        fileChooser.setMultiSelectionEnabled(false);
        fileChooser.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                final String absolutePath = f.getAbsolutePath().toUpperCase();
                return (f.isFile() && absolutePath.endsWith("SCENE")) || f.isDirectory();
            }
            public String getDescription() {
                return "Raytracer scenes (*.scene)";
            }
        });
        int returnVal = fileChooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            file = fileChooser.getSelectedFile();
            reload();
        }
    }

    private void save() {
        JFileChooser fileChooser = new JFileChooser(imageFile);
        fileChooser.setMultiSelectionEnabled(false);
        fileChooser.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                final String absolutePath = f.getAbsolutePath().toUpperCase();
                return (f.isFile() && (absolutePath.endsWith("JPG") || absolutePath.endsWith("JPEG")))
                        || f.isDirectory();
            }
            public String getDescription() {
                return "JPEG Images (*.jpg, *.jpeg)";
            }
        });
        int returnVal = fileChooser.showSaveDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            imageFile = fileChooser.getSelectedFile();
            renderer.saveImage(imageFile);
            addConsoleText("Image "+imageFile.getName()+" saved", Constants.commentColor);
        }
    }

    /**
     * Invoked when an action occurs.
     */
    synchronized public void actionPerformed(ActionEvent ev) {
        if (renderingRunning()) {
            // stop rendering
            renderer.cancelRendering();
            synchronized (synchObj) {
                while(renderingRunning())
                    try {
                        wait();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
            }
        }

        // decode size string
        String sizeString = ((String)sizeCombo.getSelectedItem()).trim();
        int width = 0;
        int height = 0;
        try {
            final StringTokenizer tkSz = new StringTokenizer(sizeString, "x");
            try {
                width = Integer.parseInt(tkSz.nextToken());
            } catch (Exception e) {
                width = 0;
            }
            try {
                height = Integer.parseInt(tkSz.nextToken());
            } catch (Exception e) {
                height = 0;
            }
        } catch (Exception e) {
            addConsoleText("decodeSize:: invalid sizeString: "+sizeString, Constants.failColor);
        }
        renderer.setSize(width, height);
        pack();
    }

    synchronized public void renderFinished(int status) {
        renderButton.setText(renderText);
        switch (status) {
            case Renderer.SUCCEEDED:
                progress.setString("Done");
                progress.setValue(100);
                pauseButton.setEnabled(false);
                break;
            case Renderer.CANCELLED:
                progress.setString("Cancelled");
                progress.setValue(100);
                pauseButton.setEnabled(false);
                break;
            case Renderer.PAUSED:
                progress.setString("Paused");
                break;
            case Renderer.NO_SCENE_DEFINED:
                progress.setString("No scene defined");
                progress.setValue(100);
                pauseButton.setEnabled(false);
                break;
        }
        notifyAll();
    }

    public void setPercent(int v) {
        progress.setValue(v);
        progress.setString(v+"%");
    }

    public static String getPath() {
        if ((frame == null) || (frame.file == null))
            return null;
        return frame.file.getParent();
    }

    public static void addConsoleText(String text, raytracer.internal.Color color) {
        if ((frame == null) || (frame.console == null))
            return;
        frame.consoleText += "<font color="+Utils.hexColor(color)+">"+text+"</font><br>";
        frame.console.setText(frame.consoleText);
    }

    public static void clearConsoleText() {
        if ((frame == null) || (frame.console == null))
            return;
        frame.consoleText = "<html>";
        addConsoleText("Raytracer SS03 V1.1", Constants.greenColor);
    }
}
