package raytracer.internal;

import raytracer.objects.Material;

/**
 *
 * Contains some systemwide constants.
 *
 */
public abstract class Constants {
    /**
     * Name is self-explaining ;-)
     */
    public final static float INFINITY = Float.MAX_VALUE;

    /**
     * Every non-surface point which is at least this epsilon away from a surface point
     * is guaranteed to be identified as such.
     */
    public final static float OUTSIDE_EPSILON = 0.0002f;

    /**
     * Used for
     * OUTSIDE_EPSILON_POW_2 = OUTSIDE_EPSILON ^ 2
     */
    public final static float OUTSIDE_EPSILON_POW_2 = OUTSIDE_EPSILON * OUTSIDE_EPSILON;

    //
    /**
     * Every light with influence < this epsilon can be ignored (for speed reasons).
     */
    public final static float LIGHT_INFLUENCE_EPSILON = 0.002f;

    /**
     * Set recursion depth to a high value so that only light influence > constant amout would be added.
     */
    public final static int MAX_RECURSION_DEPTH = 100;

    /**
     * A material to deliver wenn no material defined.
     */
    public static final Material standardMaterial = Material.yellow();

    public static final Color greenColor = new Color(.2f,.8f,.2f);
    public static final Color failColor = new Color(1,0,0);
    public static final Color commentColor = new Color(0,0,1);
}
