package raytracer.internal;

import raytracer.objects.Material;
import raytracer.objects.Surface;

public class Hit {
    public static Hit NO_INTERSECTION = new Hit();

    private Ray ray = null;
    private Surface surface = null;
    private float t = Constants.INFINITY;
    private Point point = null;
    private Vector normal = null;
    private boolean entering = true;
    private Material material = Constants.standardMaterial;

    private Hit() {
        final Vector v = new Vector(0,0,0);
        final Point p = new Point(0,0,0);
        this.ray = new Ray(p, v);
        this.surface = null;
        this.t = Constants.INFINITY;
        this.point = p;
        this.normal = v;
    }

    public Hit(Ray ray, Surface surface, float t, Point point, Vector normal, boolean entering, Material material) {
        this.ray = ray;
        this.surface = surface;
        this.t = t;
        this.point = point;
        this.normal = normal;
        this.entering = entering;
        this.material = material;
    }

    public Hit(Ray ray, Surface surface, float t, boolean entering, Material material) {
        this.ray = ray;
        this.surface = surface;
        this.t = t;
        this.entering = entering;
        this.material = material;
    }

    public Point getPoint() {
        if (point == null) {
            point = ray.pointAt(t);
            entering = surface.isInside(ray.pointAt(t+2f*Constants.OUTSIDE_EPSILON));
        }
        return point;
    }

    public Vector getNormal() {
        if (normal == null)
            normal = surface.calcNormal(this);
        return normal;
    }

    public final Surface getSurface() {
        return surface;
    }

    public void setSurface(Surface surface) {
        this.surface = surface;
        this.normal = null;
        this.point = null;
    }

    public final float getT() {
        return t;
    }

    public void setT(float t) {
        this.t = t;
        this.normal = null;
        this.point = null;
    }

    public void setEntering(boolean entering) {
        this.entering = entering;
    }

    public final Ray getRay() {
        return ray;
    }

    public boolean entering() {
        if (point == null)
            getPoint();
        return entering;
    }

    public Material getMaterial() {
        if (material == null)
            return Constants.standardMaterial;
        return material;
    }

    public boolean hasPositiveT() {
        return t > Constants.OUTSIDE_EPSILON;
    }

    public void setMaterial(Material material) {
        this.material = material;
    }

	public Color getTextureColor() {
		if (surface != null)
			return getMaterial().getTextureColor(surface.calcTextureParam(getPoint()));
		return Color.white;
	}
}
