package raytracer.internal;

/**
 *
 * Representation of a ray
 *
 */
public class Ray {
    public Point origin;
    public Vector direction;

    /**
     * Creates a ray
     * @param p Point where the ray starts
     * @param v Direction vector of the ray
     */
    public Ray(Point p, Vector v) {
        this.origin = p;
        this.direction = v.normalize();
    }

    /**
     * Returns the point which is getT units away (in the ray direction) from the ray starting point
     * @param t Number of units
     * @return New point with the requested distance
     */
    public Point pointAt(float t) {
        return origin.add(direction.scale(t));
    }

    /**
     * Returns a new ray with the same direction but an "advanced" starting point (to avoid numeric problems)
     * @param t Number of units to shift
     * @return New ray with shifted starting point
     */
    public Ray advance(float t) {
        return new Ray(pointAt(t), direction);
    }
}
