package raytracer.internal;

/**
 * Created by IntelliJ IDEA.
 * User: Bogdy
 * Date: 09.05.2003
 * Time: 16:27:46
 * To change this template use Options | File Templates.
 */
public class Utils {
    public static final boolean sameFloat(float a, float b) {
        return Math.abs(a-b) < Constants.OUTSIDE_EPSILON;
    }

    /**
     * @param floats Sorted array of intersection distances
     * @return Return first value bigger than 0, Constants.INFINITY if floats == null
     */
    public static final Hit firstPositiveValue(Hit[] floats) {
        if (floats == null)
            return Hit.NO_INTERSECTION;
        for (int i = 0; i < floats.length; i++)
            if (floats[i].hasPositiveT())
                    return floats[i];
        return Hit.NO_INTERSECTION;
    }

	/**
	 * @param x
	 * @param n
	 * @return x % n also for negative x values 
	 */
	public static int modulo(int x, int n) {
		if (x > 0)
			return x % n;
		return (x+(-x/n + 1)*n) % n;
	}

	/**
	 * @param f
	 * @return signature of f: -1, 0, 1
	 */
	public static float sgn(final float f) {
		if (sameFloat(f, 0))
			return 0;
		return f / Math.abs(f);
	}

    public static String hexColor(Color color) {
        if (color == null)
            return "000000";

        // remove alpha channel
        final int rgb = color.getRGBAsInt();

        // fill with leading zeros
        String hexStr = Integer.toHexString(rgb);
        while(hexStr.length() < 6)
            hexStr = "0"+hexStr;
        return hexStr;
    }
}
