package raytracer.objects;

import raytracer.internal.Hit;
import raytracer.internal.TexturePoint;
import raytracer.internal.Transformation;
import raytracer.internal.Utils;
import raytracer.internal.Vector;
import raytracer.internal.Point;

import java.util.ArrayList;


public abstract class Compositum extends Surface {

    // use own material or that of the siblings
    protected boolean useSiblingMaterial = false;
    protected Surface firstSurface = null;
    protected Surface secondSurface = null;

    public void setSiblingmaterial(float t) {
        useSiblingMaterial = !Utils.sameFloat(t, 0);
    }

    public boolean getUseSiblingMaterial() {
        return useSiblingMaterial;
    }

    public void setFirst(Surface firstSurface) {
        this.firstSurface = firstSurface;
    }

    public void setSecond(Surface secondSurface) {
        this.secondSurface = secondSurface;
    }

    /**
     * Calculates the surface normal at the given point
     * @param hit Point to calculate the normal at
     * @return Normal vector in point origin
     */
    public Vector calcNormal(Hit hit) {
        return null;
    }

    /**
     * Applies a given transformation to this object
     * @param t Transformation (matrix)
     * @return Should always return "this"
     */
    public Surface apply(Transformation t) {
        firstSurface.apply(t);
        secondSurface.apply(t);
        return this;
    }

    protected Hit[] returnColoredHits(ArrayList list) {
        final Hit[] hits = (Hit[])list.toArray(new Hit[list.size()]);

        // apply parent color
        if (!useSiblingMaterial)
            for (int i = 0; i < hits.length; i++) {
                hits[i].setMaterial(material);
            }

        return hits;
    }

	/* (non-Javadoc)
	 * @see raytracer.objects.Surface#calcTextureParam(raytracer.internal.Point)
	 */
	public TexturePoint calcTextureParam(Point p) {
		return null;
	}

    public NamedObject setAttributes(NamedObject newObject) {
        super.setAttributes(newObject);

        final Compositum compositum = (Compositum)newObject;
        compositum.useSiblingMaterial = useSiblingMaterial;
        compositum.firstSurface = null;
        compositum.secondSurface = null;
        if (firstSurface != null)
            compositum.firstSurface = (Surface)firstSurface.duplicate();
        if (secondSurface != null)
            compositum.secondSurface = (Surface)secondSurface.duplicate();
        return newObject;
    }
}
