package raytracer.objects;

import raytracer.internal.*;

/**
 * Created by IntelliJ IDEA.
 * User: Bogdy
 * Date: 18.05.2003
 * Time: 19:43:25
 * To change this template use Options | File Templates.
 */
public class Group extends Surface {

    protected java.util.Vector children = new java.util.Vector();
    private boolean copyContent = false;

    public Group() {
        this.copyContent = false;
        children.removeAllElements();
    }

    protected Group(boolean copy) {
        this.copyContent = copy;
        children.removeAllElements();
    }

    public void setSurface(Surface object) {
        if (copyContent)
            children.add(object.duplicate());
        else
            children.add(object);
    }

    public Surface[] getPrimitiveSurfaces() {
        java.util.Vector primitiveChildren = new java.util.Vector();
        for (int i = 0; i < children.size(); i++) {
            Surface surface = (Surface) children.elementAt(i);
            System.out.println("child "+i+" "+surface.getName());
            if ((surface instanceof Group) || (surface instanceof GroupCopy)) {
                final Surface[] primitiveSurfaces = ((Group)surface).getPrimitiveSurfaces();
                for (int j = 0; j < primitiveSurfaces.length; j++) {
                    Surface primitiveSurface = primitiveSurfaces[j];
                    System.out.println("group prim "+j+" "+primitiveSurface.getName());
                    primitiveChildren.add(primitiveSurface);
                }
            } else {
                primitiveChildren.add(surface);
            }
        }
        return (Surface[])primitiveChildren.toArray(new Surface[primitiveChildren.size()]);
    }

    /**
     * Calculate the first intersection of a given ray with this object
     * @param ray Ray to calculate intersection with
     * @return Distance between ray starting point and intersection point along the ray direction,
     * INFINITY if no intersection occours.
     * Negative lengths are ignored.
     */
    public Hit intersectRay(Ray ray) {
        return Hit.NO_INTERSECTION;
    }

    /**
     * Calculate all intersections of a given ray with this object
     * @param ray Ray to calculate intersection with
     * @return Array of distances between ray starting point and intersection point
     * along the ray direction, null if no intersection occours.
     * Negative lengths included.
     */
    public Hit[] intersect(Ray ray) {
        return null;
    }

    /**
     * Determines whether a given point is inside this object or not
     * @param p Query point
     * @return true, if origin is inside this object, otherwise false
     */
    public boolean isInside(Point p) {
        return false;
    }

    /**
     * Calculates the surface normal at the given point
     * @param hit Point to calculate the normal at
     * @return Normal vector in point origin
     */
    public Vector calcNormal(Hit hit) {
        return null;
    }

    /**
     * Applies a given transformation to this object
     * @param t Transformation (matrix)
     * @return Should always return "this"
     */
    public Surface apply(Transformation t) {
        for (int i = 0; i < children.size(); i++) {
            ((Surface) children.elementAt(i)).apply(t);
        }
        return this;
    }

    /**
     * @param p
     * @return texture coordinates mapped to the point p
     */
    public TexturePoint calcTextureParam(Point p) {
        return null;
    }

    public NamedObject duplicate() {
        return setAttributes(new Group());
    }

    public NamedObject setAttributes(NamedObject newObject) {
        super.setAttributes(newObject);

        //Set duplicate of children
        final Group group = (Group)newObject;
        group.children.removeAllElements();
        for (int i = 0; i < children.size(); i++) {
            Surface surface = (Surface)children.elementAt(i);
            if (surface != null)
                group.children.add(surface.duplicate());
        }
        return newObject;
    }
}
