package raytracer.objects;

import raytracer.internal.Color;
import raytracer.internal.TexturePoint;

public class Material extends NamedObject {

    private Texture texture;
	private Color diffuseColor, specularColor, reflectionColor, transmissionColor;
    private float refractionIndex, specularExponent;

    private static Material yellow;

    public static Material yellow() {
        if (yellow == null) {
            yellow = new Material();
            yellow.setSpecular(1, 1, 0.4f, 4);
            yellow.setDiffuse(1, 1, 0);
            yellow.setReflection(0, 0, 0);
            yellow.setTransmission(0, 0, 0, 1);
        }
        return yellow;
    }

    public Material() {
        diffuseColor = specularColor = reflectionColor = transmissionColor = new Color(0, 0, 0);
        texture = null;
        refractionIndex = 1;
        specularExponent = 16;
    }

	public void setDiffuse(float r, float g, float b) {
        diffuseColor = new Color(r, g, b);
	}

	public void setSpecular(float r, float g, float b, float exp) {
        specularColor = new Color(r, g, b);
        specularExponent = exp;
	}

	public void setReflection(float r, float g, float b) {
        reflectionColor = new Color(r, g, b);
	}

	public void setTransmission(float r, float g, float b, float idx) {
        transmissionColor = new Color(r, g, b);
        // @todo is refractionIndex == idx
        refractionIndex = idx;
	}

	public void setTexture(Texture texture) {
		this.texture = texture;
	}

	public Color getDiffuseColor() {
        return diffuseColor;
    }

	public Color getReflectionColor() {
        return reflectionColor;
    }

	public float getRefractionIndex() {
        return refractionIndex;
    }

	public Color getSpecularColor() {
        return specularColor;
    }

	public float getSpecularExponent() {
        return specularExponent;
    }

	public Color getTransmissionColor() {
        return transmissionColor;
    }

	public Color getTextureColor(TexturePoint texPt) {
		if (texture == null)
			return Color.white;
//		texPt.print();
		return texture.getColor(texPt);
	}

    public NamedObject setAttributes(NamedObject newObject) {
        final Material material = (Material)newObject;
        material.setDiffuse(diffuseColor.r, diffuseColor.g, diffuseColor.b);
        material.setSpecular(specularColor.r, specularColor.g, specularColor.b, specularExponent);
        material.setReflection(reflectionColor.r, reflectionColor.g, reflectionColor.b);
        material.setTransmission(transmissionColor.r, transmissionColor.g, transmissionColor.b, refractionIndex);
        if (texture != null)
            material.setTexture((Texture)texture.duplicate());
        return newObject;
    }

    /**
     * @return a copy of the current object
     */
    public NamedObject duplicate() {
        return setAttributes(new Material());
    }
}
