/*
 * Created on 14.05.2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package raytracer.objects;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import raytracer.ControlFrame;
import raytracer.internal.Color;
import raytracer.internal.TexturePoint;
import raytracer.internal.Utils;

/**
 * @author Bogdy
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class Texture extends NamedObject {
	private float du = 0;
	private float dv = 0;
	private int width = 0;
	private int height = 0;
	private TexturePoint startPoint = new TexturePoint(0,0);
	private TexturePoint endPoint = new TexturePoint(1, 1);
	private BufferedImage image = null;

    public Texture() {
    }

	public void setStartPoint(float u, float v) {
		startPoint = new TexturePoint(u, v);
		recompute();
	}

	public void setEndPoint(float u, float v) {
		endPoint = new TexturePoint(u, v);
		recompute();
	}
	
	public void setImage(String fileName) {
		//System.out.println("file "+fileName);
		image = null;
		if (fileName == null)
			return;

		try {
			image = ImageIO.read(new File(ControlFrame.getPath(), fileName));
		} catch (IOException e) {
			System.out.println("Could not load texture file "+fileName+" !");
			image = null;
		}
		recompute();
	}

	private void recompute() {
		final float u = endPoint.getU() - startPoint.getU();
		final float v = endPoint.getV() - startPoint.getV();
		if ((image == null) || Utils.sameFloat(u, 0) || Utils.sameFloat(v, 0)) {
			width = 0;
			height = 0;
			du = 0;
			dv = 0;
		} else {
			width = image.getWidth();
			height = image.getHeight();
			du = (width*1f) /u;
			dv = (height*1f)/v;
		}
	}

	public Color getColor(TexturePoint texPt) {
		if ((image == null) || (texPt == null))
			return Color.white;

        final int x = Utils.modulo((int)((texPt.getU() - startPoint.getU()) * du), width);
        final int y = Utils.modulo((int)((texPt.getV() - startPoint.getV()) * dv), height);

		return new Color(image.getRGB(x, y));
	}

    public NamedObject setAttributes(NamedObject newObject) {
        final Texture texture = (Texture)newObject;
        texture.setStartPoint(startPoint.getU(), startPoint.getV());
        texture.setEndPoint(endPoint.getU(), endPoint.getV());
        texture.image = image;
/*
        // don't copy the image
        if (image != null) {
            texture.image = new BufferedImage(image.getWidth(), image.getHeight(), image.getType());
            texture.image.setData(image.getData());
        }
*/
        texture.recompute();
        return newObject;
    }

    /**
     * @return a copy of the current object
     */
    public NamedObject duplicate() {
        return setAttributes(new Texture());
    }
}
