package raytracer.objects;

import raytracer.internal.Hit;
import raytracer.internal.Point;
import raytracer.internal.Ray;
import raytracer.internal.Utils;

import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: Bogdy
 * Date: 08.05.2003
 * Time: 00:04:21
 * To change this template use Options | File Templates.
 */
public class Union extends Compositum {

    public Union() {
        super();
    }

    /**
     * Calculate the first intersection of a given ray with this object
     * @param ray Ray to calculate intersection with
     * @return Distance between ray starting point and intersection point along the ray direction, INFINITY if no intersection occours
     */
    public Hit intersectRay(Ray ray) {
        return Utils.firstPositiveValue(intersect(ray));
    }

    /**
     * Calculate all intersections of a given ray with this object
     * @param ray Ray to calculate intersection with
     * @return Array of distances between ray starting point and intersection point
     * along the ray direction, null if no intersection occours.
     * Negative lengths included.
     */
    public Hit[] intersect(Ray ray) {
        final Hit[] hitFirst = firstSurface.intersect(ray);
        final Hit[] hitSecond = secondSurface.intersect(ray);

        // if none or only one object hit
        if (hitFirst == null)
            return hitSecond;
        if (hitSecond == null)
            return hitFirst;

        // if both objects were hit
        ArrayList list = new ArrayList();

        int i1 = 0;
        int i2 = 0;
        boolean open1 = false;
        boolean open2 = false;
        while ((i1 < hitFirst.length) || (i2 < hitSecond.length)) {
            if ((i1 < hitFirst.length) && (i2 >= hitSecond.length))
                while (i1 < hitFirst.length)
                    list.add(hitFirst[i1++]);
            else if ((i1 >= hitFirst.length) && (i2 < hitSecond.length))
                while (i2 < hitSecond.length)
                    list.add(hitSecond[i2++]);
            else if (hitFirst[i1].getT() < hitSecond[i2].getT()) {
                if ((!open1 && !open2) || (open1 && !open2))
                    list.add(hitFirst[i1]);
                i1++;
                open1 = !open1;
            } else {
                if ((!open1 && !open2) || (!open1 && open2))
                    list.add(hitSecond[i2]);
                i2++;
                open2 = !open2;
            }
        }

        // no intersection
        if (list.size() == 0)
            return null;

        return returnColoredHits(list);
    }

    /**
     * Determines whether a given point is inside this object or not
     * @param p Query point
     * @return true, if origin is inside this object, otherwise false
     */
    public boolean isInside(Point p) {
        return firstSurface.isInside(p) || secondSurface.isInside(p);
    }

    public NamedObject duplicate() {
        return setAttributes(new Union());
    }
}
