package raytracer.tests.internal;

/**
 * Created by IntelliJ IDEA.
 * User: Bogdy
 * Date: 02.05.2003
 * Time: 13:49:10
 * To change this template use Options | File Templates.
 */

import junit.framework.TestCase;
import raytracer.internal.Matrix;
import raytracer.internal.Transformation;
import raytracer.internal.Point;

public class MatrixTest extends TestCase {
    private Matrix a = null;
    private Matrix b = null;
    private Matrix c = null;
    private Matrix mulBC = null;
    private Matrix mulCB = null;
    private Matrix mulBCt = null;
    private Matrix mulCtB = null;
    private Matrix invB = null;
    private float detC = 0;
    private float detB = 0;
    private Point p = null;
    private Point p2 = null;

    public MatrixTest(String s) {
        super(s);
    }

    protected void setUp() {
        a = new Matrix();
        b = new Matrix(
                3, 5, 8, 7,
                1, 6, 5, 2,
                -1, 5, 6, 9,
                0, 3, 2, 2);
        c = new Matrix(
                2, 3, 0, 1,
                -1, 0, 0, 2,
                3, 0, 1, -1,
                4, 0, 0, 1);
        detC = 27;
        detB = 88;
        invB = new Matrix(
                0.3750f,   -0.5682f,   -0.4432f,    1.2500f,
                0.0000f,   -0.1818f,   -0.1818f,    1.0000f,
                -0.1250f,    0.7045f,    0.3295f,   -1.7500f,
                0.1250f,   -0.4318f,   -0.0568f,    0.7500f);
        mulBC = new Matrix(
                53, 9, 8, 12,
                19, 3, 5, 10,
                47, -3, 6, 12,
                11, 0, 2, 6);
        mulCB = new Matrix(
                9, 31, 33, 22,
                -3, 1, -4, -3,
                8, 17, 28, 28,
                12, 23, 34, 30);
        mulBCt = new Matrix(
                28,    11,    10,    19,
                22,     3,     6,     6,
                22,    19,    -6,     5,
                11,     4,     0,     2);
        mulCtB = new Matrix(
                2,    31,    37,    47,
                9,    15,    24,    21,
                -1,     5,     6,     9,
                6,    15,    14,     4);

        p = new Point(1, 0, 0);
        p2 = new Point(3, -2, 1);
    }

    public void testConstructor() {
        assertNotNull("matrix not initialized", a);
        assertNotNull("matrix not initialized", b);
        assertNotNull("matrix not initialized", c);
        assertNotNull("matrix not initialized", mulBC);
        assertNotNull("matrix not initialized", mulCB);
    }

    public void testMul() {
        a.unify();
        assertTrue("mulRight", a.mulRight(b).same(b));
        assertTrue("mulLeft", a.mulLeft(b).same(b));

        assertTrue("mulRight: BC", b.mulRight(c).same(mulBC));
        assertTrue("mulLeft: CB", b.mulLeft(c).same(mulCB));

        assertTrue("mulRight: BCt", b.mulRightTrans(c).same(mulBCt));
        assertTrue("mulLeft: CtB", b.mulLeftTrans(c).same(mulCtB));
    }

    public void testInv() {
        final float detc = c.det();
        final float detb = b.det();
        assertTrue("det C="+detc, detc == detC);
        assertTrue("det B="+detb, detb == detB);
        final Matrix inversB = b.inverse();
        assertTrue("inv B", inversB.same(invB));
    }

    public void testRotation() {
        assertNotNull("point undefined", p);
        Transformation t = new Transformation();
        t.setMatrix(Matrix.getRotationMatrix(0, 0, 1, (float)Math.PI/2f));
        assertTrue("rotation failed", p.transform(t).same(new Point(0, 1, 0)));
        t.setMatrix(Matrix.getRotationMatrix(0, 0, 1, (float)Math.PI));
        assertTrue("rotation failed", p.transform(t).same(new Point(-1, 0, 0)));
    }

    public void testTranslation() {
        assertNotNull("point undefined", p);
        Transformation t = new Transformation();
        t.setMatrix(Matrix.getTranslationMatrix(2, 4, 1));
        t.getMatrix().print();
        assertTrue("translation failed", p.transform(t).same(new Point(3, 4, 1)));
        t.setMatrix(Matrix.getTranslationMatrix(-3, 1, -2));
        t.getMatrix().print();
        assertTrue("translation failed", p.transform(t).same(new Point(-2, 1, -2)));
    }

    public void testScale() {
        assertNotNull("point undefined", p2);
        Transformation t = new Transformation();
        t.setMatrix(Matrix.getScaleMatrix(2, .4f, 1));
        t.getMatrix().print();
        assertTrue("translation failed", p2.transform(t).same(new Point(6, -.8f, 1)));
        t.setMatrix(Matrix.getScaleMatrix(-.3f, 1, -2f));
        t.getMatrix().print();
        assertTrue("translation failed", p2.transform(t).same(new Point(-.9f, -2, -2)));
    }

/*    protected void runTest() {

}
*/
    protected void tearDown() {
    }
}
