package raytracer.tests.objects;

/**
 * Created by IntelliJ IDEA.
 * User: Bogdy
 * Date: 02.05.2003
 * Time: 13:49:10
 * To change this template use Options | File Templates.
 */

import junit.framework.TestCase;
import raytracer.internal.*;
import raytracer.objects.Sphere;
import raytracer.objects.Material;

public class SphereTest extends TestCase {
    private Sphere sphere = null;

    public SphereTest(String s) {
        super(s);
    }

    protected void setUp() {
        sphere = new Sphere();
        sphere.setCenter(0, 0, 1);
        sphere.setRadius(0.5f);
    }

    public void testConstructor() {
        assertNotNull("sphere not initialized", sphere);
    }

    public void testRayIntersect() {
        assertNotNull("sphere not initialized", sphere);
        Ray ray = new Ray(new Point(0,0,0), new Vector(0,0,1));
        assertNotNull("ray not initialized", ray);
        Hit t = sphere.intersectRay(ray);
        assertEquals("intersection result failed", 0.5f, t.getT(), Constants.OUTSIDE_EPSILON);
    }

    public void testInside() {
        assertNotNull("sphere not initialized", sphere);
        boolean inside1 = sphere.isInside(new Point(0,0,1.3f));
        boolean inside2 = sphere.isInside(new Point(0,0,1.7f));
        assertTrue("inside failed", inside1);
        assertFalse("inside failed", inside2);
    }

    public void testNormal() {
        assertNotNull("sphere not initialized", sphere);
        Ray ray = new Ray(new Point(0,0,0), new Vector(0,0,1));
        Vector normal = new Hit(ray, sphere, .5f, true, sphere.getMaterial()).getNormal();
        assertTrue("calc normal failed", normal.same(new Vector(0,0,-1)));
    }

    public void testTexture() {
		assertNotNull("sphere not initialized", sphere);

// part 1
		TexturePoint tp0 = sphere.calcTextureParam(new Point(0,0,0.5f));
		tp0.print();
		assertTrue("calcTextureParam failed", tp0.same(new TexturePoint(0, 1)));

		TexturePoint tp1 = sphere.calcTextureParam(new Point(0,0.5f,1));
		tp1.print();
		assertTrue("calcTextureParam failed", tp1.same(new TexturePoint(.25f, .5f)));

		TexturePoint tp2 = sphere.calcTextureParam(new Point(0.5f,0,1));
		tp2.print();
		assertTrue("calcTextureParam failed", tp2.same(new TexturePoint(0, .5f)));

//		part 2
		TexturePoint tp3 = sphere.calcTextureParam(new Point(0,0,1.5f));
		tp3.print();
		assertTrue("calcTextureParam failed", tp3.same(new TexturePoint(0, 0)));

		TexturePoint tp4 = sphere.calcTextureParam(new Point(0,-0.5f,1));
		tp4.print();
		assertTrue("calcTextureParam failed", tp4.same(new TexturePoint(.75f, .5f)));

		TexturePoint tp5 = sphere.calcTextureParam(new Point(-0.5f,0,1));
		tp5.print();
		assertTrue("calcTextureParam failed", tp5.same(new TexturePoint(0, .5f)));

//		part 3 - between
		TexturePoint tp6 = sphere.calcTextureParam(new Point(0,0.5f,1.5f));
		tp6.print();
		assertTrue("calcTextureParam failed", tp6.same(new TexturePoint(0.25f, 0.25f)));

		TexturePoint tp7 = sphere.calcTextureParam(new Point(0,-0.5f,1.5f));
		tp7.print();
		assertTrue("calcTextureParam failed", tp7.same(new TexturePoint(.75f, .25f)));

		TexturePoint tp8 = sphere.calcTextureParam(new Point(-0.5f,0,1.5f));
		tp8.print();
		assertTrue("calcTextureParam failed", tp8.same(new TexturePoint(.5f, .25f)));

		TexturePoint tp9 = sphere.calcTextureParam(new Point(0.5f,0,1.5f));
		tp9.print();
		assertTrue("calcTextureParam failed", tp9.same(new TexturePoint(0, .25f)));
    }

/*    protected void runTest() {

    }
    */
    protected void tearDown() {
    }
}
